#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include <libexif/exif-data.h>
#include <libexif/exif-utils.h>
#include <libexif/exif-loader.h>

#ifndef __linux__
#include <windows.h>
#endif

void print_usage() {
	printf("Verwendung: rts [bild bild.jpg | datei datei.dat]\n");
	printf("   Bei Angabe eines Bildes: Umbenennen nach EXIF_TAG_DATE_TIME_ORIGINAL\n");
	printf("   Bei Angabe einer  Datei: Umbenennen nach Datei-Zeitstempel\n");
}

void rename_file(char *filename, char *timeStr) {
	int i=0;
	char *pch, s[200], s2[200];
#ifndef __linux__
	// hole Pointer auf letztes Vorkommen eines Backslashes
	pch = strrchr(filename, '\\');
#else
	// hole Pointer auf letztes Vorkommen eines Slashes
	pch = strrchr(filename, '/');
#endif
	// falls Backslash vorhanden
	if (pch != NULL) {
		// Stelle des Backslashes berechnen
		i = pch-filename+1;
		// String ab Backslash bis Ende in s kopieren
		strncpy(s, filename+i, strlen(filename));
		// zwischen Zeit und Dateiname noch ein Minus
		sprintf(s2, "%s - %s", timeStr, s);
	}
	// falls kein Backslash vorhanden
	else {
		printf("\nAchtung: Kein ganzer Pfad angegeben.\n");
		sprintf(s2, "%s - %s", timeStr, filename);
	}
	printf("\nIch werde jetzt umbenennen:\n%s\nin\n%s\n", filename, s2);
	rename(filename, s2);
	printf("\nOk Datei umbenannt, beende Programm.\n");
}

int main(int argc, char *argv[]) {
	// Anzahl der Argumente auf mind. 2 ueberpruefen
	if (argc <= 2) {
		print_usage();
		return 0;
	}
	// ein paar Variablen...
	int i = 0;
	char *filename = argv[2], timeStr[200];
	struct stat buf;
	// erstes Argument = "bild"
	if (strcmp(argv[1], "bild") == 0) {
		char year[5], month[5], day[5], hour[5], minute[5], second[5], mytimestamp[20];
		ExifData *ed;
		ExifIfd ifd = EXIF_IFD_EXIF;
		ExifTag tag = EXIF_TAG_DATE_TIME_ORIGINAL;
		ed = exif_data_new_from_file(argv[2]);
		if (!ed) {
			printf("Keine Bilddatei angegeben oder keine EXIF-Daten vorhanden.\n\n");
			print_usage();
			return 1;
		}
		ExifEntry *entry = exif_content_get_entry(ed->ifd[ifd], tag);
		if (entry) {
			char buff[1024];
			exif_entry_get_value(entry, buff, sizeof(buff));
			sprintf(year, "%c%c%c%c", buff[0], buff[1], buff[2], buff[3]);
			sprintf(month, "%c%c", buff[5], buff[6]);
			sprintf(day, "%c%c", buff[8], buff[9]);
			sprintf(hour, "%c%c", buff[11], buff[12]);
			sprintf(minute, "%c%c", buff[14], buff[15]);
			sprintf(second, "%c%c", buff[17], buff[18]);
			sprintf(mytimestamp, "%s-%s-%s-%s-%s-%s", year, month, day, hour, minute, second);
			printf("%s: %s\nDateTimeOriginal: %s\n", exif_tag_get_name_in_ifd(tag, ifd), buff, mytimestamp);
		}
		exif_data_unref(ed);
		// beschreibe Struktur mit Datei-Daten
		if (!stat(filename, &buf)) {
			// vorher bestimmten EXIF-timestamp in timeStr schreiben
			sprintf(timeStr, "%s", mytimestamp);
			// Funktionsaufruf zum Umbenennen
			rename_file(filename, timeStr);
		} else {
			// Fehlerfall
			printf("\nEtwas stimmt nicht. Datei existiert nicht?\n\n");
			print_usage();
		}
	// erstes Argument = "datei"
	} else if (strcmp(argv[1], "datei") == 0) {
		// beschreibe Struktur mit Datei-Daten
		if (!stat(filename, &buf)) {
			// befuelle timeStr mit formatierter Zeitangabe
			strftime(timeStr, 200, "%Y-%m-%d - %H-%M-%S", localtime(&buf.st_ctime));
			// Funktionsaufruf zum Umbenennen
			rename_file(filename, timeStr);
		} else {
			// Fehlerfall
			printf("\nEtwas stimmt nicht. Datei existiert nicht?\n\n");
			print_usage();
		}
		//printf("\n\nBeliebige Taste druecken zum Beenden...\n");
		//char c = getchar();
		return 0;
	// erstes Argument ist weder "bild" noch "datei"
	} else {
		printf("Sonst was falsch gemacht.\n");
		return 0;
	}
	return 0;
}
